#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import math
import pygame
pygame.init()

from code.other.helpers import load_image
from code.classes.Animated_Sprite import Animated_Sprite


class Enemy( Animated_Sprite ):
	"""
	This is the base class for enemies.  There are 3 types of enemies:

	Static enemies:  These enemies can shoot, but not move.  Usually, these are towers.
	Moving enemies:	 These enemies can shoot and move, thus, they can follow the player.  Usually spaceships or rovers.
	Kamikaze enemies:These enemies can't shoot but move.  They always move towards the players spaceship and try, to destroy / damage it.
	"""

	def __init__( self, image_file_names_enemy, animation_intervals_enemy, health_points, projectile_damage, name, left_top_position, detection_distance ):
		"""
		Parameters
		"""

		super( Enemy, self ).__init__( image_file_names_enemy, animation_intervals_enemy )
		self.rect = self.image.get_rect( topleft = left_top_position )

		self._health_points = health_points
		self._projectile_damage = projectile_damage
		self._name = name
		self._left_top_position = left_top_position

		self._distance = 0 # The distance to the lunar rover will be set in the game.
		self._detection_distance = detection_distance # At which max distance towards the lunar rover will the enemy wake up e.g. attack him?
		self._sleep = True


	def update( self, moon_rover_xy_velocity ):
		"""
		Parameters
			moon_rover_xy_velocity
				Defines where the enemy should be moved to when the player moves with his lunar rover.
		"""

		self.rect.move_ip( moon_rover_xy_velocity[0], moon_rover_xy_velocity[1] )	

		# And update the animation.
		self.update_animation()

		# Should the enemy start to move and fire shots at the lunar rover or not ?
		self.sleep_or_wakeup()		


	def reduce_health_points( self, shot_damage ):
		"""
		Reduces the health points of the enemy object.
		"""

		self._health_points -= shot_damage

		# Check if the enemy is still alive after it has been hit by the shot.
		# If not, then remove that enemy.
		if not self.check_health_points():
			self.kill()


	def check_health_points( self ):
		"""
		Function that checks if the health points are more than 0 points.

		If so, then 'True' is returned; otherwise, 'False' is returned.
		"""

		if self._health_points > 0:
			return True
		return False


	def calculate_distance_to_lunar_rover( self, lunar_rover_center ):
		"""
		Calculates the distance to the lunar rover and returns 'True' if the distance to the lunar rover is too small.
		Otherwise, 'False' is returned
		"""

		# I store the center coords of the lunar rover.
		lr_center = lunar_rover_center


		# I store the center coords of the moving enemy.
		me_center = ( self.rect.centerx, self.rect.centery )


		# Then I subtract me_center from lr_center.
		x = abs( lr_center[0] ) - me_center[0]
		y = abs( lr_center[1] ) - me_center[1]


		# I calculate the hypot.
		hypot = math.hypot( x, y )

		# And this is the distance in px.
		distance = hypot

		self._distance = distance


	def sleep_or_wakeup( self ):
		"""
		Sets self._sleep to 'False', if the player is within the enemies detection area.
		Otherwise, it will be set to 0 and the enemy sleeps again.
		"""

		if self._distance <= self._detection_distance:
			self._sleep = False
		else:
			self._sleep = True		
