#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import pygame
pygame.init()

from code.other.helpers import load_image


class Menu_Button( pygame.sprite.Sprite ):
	"""
	A simple button that uses graphics.
	"""

	def __init__( self, btn_image_off_filename, btn_image_on_filename, action, left_top_position ):
		"""
		Parameters
			btn_image_off_filename		string
				The filename of the image to be used for the button if the button is not highlightend.
			btn_image_on_filename		string
				The filename of the image to be used for the button if the button is highlightend.
			action						string
				A string that tells what to do if the button is clicked.
			left_top_position			( int, int ) or [ int, int ]
				The position of the button on the window.
		"""
		pygame.sprite.Sprite.__init__( self )

		# I store some passed parameters
		self._action = action


		# I create the sprites for the button.
		self._btn_off = pygame.sprite.Sprite()
		self._btn_off.image = load_image( btn_image_off_filename )
		self._btn_off.rect = self._btn_off.image.get_rect( topleft = left_top_position )

		self._btn_on = pygame.sprite.Sprite()
		self._btn_on.image = load_image( btn_image_on_filename )
		self._btn_on.rect = self._btn_on.image.get_rect( topleft = left_top_position )


		# At the beginning, the button is deactivated.
		self.dont_highlight()

		# A variable that is used to check if a button is highlightend.
		self._is_active = False
		self._clicked = False


	def update( self ):

		if self.mouse_over_button() and not self._is_active:
			self.highlight()
			self._is_active = True
		elif not self.mouse_over_button() and self._is_active:
			self.dont_highlight()
			self._is_active = False

		if self._is_active and pygame.mouse.get_pressed()[0]:
			# The button has been clicked.
			self._clicked = True
		else:
			self._clicked = False

	def highlight( self ):
		"""
		Highlights the button.  This happens if the mouse-pointer is above the button.
		"""

		self.image = self._btn_on.image
		self.rect = self._btn_on.rect


	def dont_highlight( self ):
		"""
		Set the button to its inactive state.  This happens if the mouse-pointer is not above the button.
		"""

		self.image = self._btn_off.image
		self.rect = self._btn_off.rect


	def mouse_over_button( self ):
		"""
		Checks if the mouse-pointer is above the button.  If so, then it returns 'True'; otherwise, 'False' is returned.
		"""

		if self.rect.collidepoint( pygame.mouse.get_pos() ):
			return True
		else:
			return False
